@@ -7,6 +7,7 @@
  * Copyright (C) 2004-2006 Elijah Newren
  * Copyright (C) 2013 Red Hat Inc.
  * Copyright (C) 2017-2019 Alberts Muktupāvels
+ * Copyright (C) 2020 NVIDIA CORPORATION
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -376,6 +377,9 @@ apply_crtc_assignments (GfMonitorManager    *manager,
       gf_crtc_unset_config (crtc);
     }
 
+  if (n_crtcs == 0)
+    goto out;
+
   g_assert (width > 0 && height > 0);
   /* The 'physical size' of an X screen is meaningless if that screen
    * can consist of many monitors. So just pick a size that make the
@@ -477,6 +481,7 @@ apply_crtc_assignments (GfMonitorManager    *manager,
       gf_output_unassign_crtc (output);
     }
 
+out:
   XUngrabServer (xrandr->xdisplay);
   XFlush (xrandr->xdisplay);
 
@@ -721,6 +726,9 @@ gf_monitor_manager_xrandr_apply_monitors_config (GfMonitorManager        *manage
 
   if (!config)
     {
+      if (!manager->in_init)
+        apply_crtc_assignments (manager, TRUE, NULL, 0, NULL, 0);
+
       gf_monitor_manager_rebuild_derived (manager, NULL);
       return TRUE;
     }
@@ -823,10 +831,17 @@ gf_monitor_manager_xrandr_get_crtc_gamma (GfMonitorManager  *manager,
   xrandr = GF_MONITOR_MANAGER_XRANDR (manager);
   gamma = XRRGetCrtcGamma (xrandr->xdisplay, (XID) gf_crtc_get_id (crtc));
 
-  *size = gamma->size;
-  *red = g_memdup2 (gamma->red, sizeof (gushort) * gamma->size);
-  *green = g_memdup2 (gamma->green, sizeof (gushort) * gamma->size);
-  *blue = g_memdup2 (gamma->blue, sizeof (gushort) * gamma->size);
+  if (size)
+    *size = gamma->size;
+
+  if (red)
+    *red = g_memdup2 (gamma->red, sizeof (gushort) * gamma->size);
+
+  if (green)
+    *green = g_memdup2 (gamma->green, sizeof (gushort) * gamma->size);
+
+  if (blue)
+    *blue = g_memdup2 (gamma->blue, sizeof (gushort) * gamma->size);
 
   XRRFreeGamma (gamma);
 }
@@ -1000,6 +1015,13 @@ gf_monitor_manager_xrandr_get_default_layout_mode (GfMonitorManager *manager)
   return GF_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL;
 }
 
+static void
+gf_monitor_manager_xrandr_set_output_ctm (GfOutput          *output,
+                                          const GfOutputCtm *ctm)
+{
+  gf_output_xrandr_set_ctm (GF_OUTPUT_XRANDR (output), ctm);
+}
+
 static void
 gf_monitor_manager_xrandr_class_init (GfMonitorManagerXrandrClass *xrandr_class)
 {
@@ -1028,6 +1050,7 @@ gf_monitor_manager_xrandr_class_init (GfMonitorManagerXrandrClass *xrandr_class)
   manager_class->get_capabilities = gf_monitor_manager_xrandr_get_capabilities;
   manager_class->get_max_screen_size = gf_monitor_manager_xrandr_get_max_screen_size;
   manager_class->get_default_layout_mode = gf_monitor_manager_xrandr_get_default_layout_mode;
+  manager_class->set_output_ctm = gf_monitor_manager_xrandr_set_output_ctm;
 }
 
 static void
