@@ -21,6 +21,7 @@
 
 #include "config.h"
 
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <math.h>
 #include <string.h>
@@ -54,7 +55,7 @@
 
 typedef struct
 {
-  GfGpu            *gpu;
+  GfBackend        *backend;
 
   GList            *outputs;
   GList            *modes;
@@ -85,7 +86,7 @@ enum
 {
   PROP_0,
 
-  PROP_GPU,
+  PROP_BACKEND,
 
   LAST_PROP
 };
@@ -377,6 +378,17 @@ get_closest_scale_factor_for_resolution (float width,
   return best_scale;
 }
 
+static void
+unset_monitor (gpointer data,
+               gpointer user_data)
+{
+  GfOutput *output;
+
+  output = GF_OUTPUT (data);
+
+  gf_output_unset_monitor (output);
+}
+
 static void
 gf_monitor_dispose (GObject *object)
 {
@@ -388,6 +400,7 @@ gf_monitor_dispose (GObject *object)
 
   if (priv->outputs)
     {
+      g_list_foreach (priv->outputs, unset_monitor, NULL);
       g_list_free_full (priv->outputs, g_object_unref);
       priv->outputs = NULL;
     }
@@ -426,8 +439,8 @@ gf_monitor_get_property (GObject    *object,
 
   switch (property_id)
     {
-      case PROP_GPU:
-        g_value_set_object (value, priv->gpu);
+      case PROP_BACKEND:
+        g_value_set_object (value, priv->backend);
         break;
 
       default:
@@ -450,8 +463,8 @@ gf_monitor_set_property (GObject      *object,
 
   switch (property_id)
     {
-      case PROP_GPU:
-        priv->gpu = g_value_get_object (value);
+      case PROP_BACKEND:
+        priv->backend = g_value_get_object (value);
         break;
 
       default:
@@ -463,11 +476,11 @@ gf_monitor_set_property (GObject      *object,
 static void
 gf_monitor_install_properties (GObjectClass *object_class)
 {
-  monitor_properties[PROP_GPU] =
-    g_param_spec_object ("gpu",
-                         "GfGpu",
-                         "GfGpu",
-                         GF_TYPE_GPU,
+  monitor_properties[PROP_BACKEND] =
+    g_param_spec_object ("backend",
+                         "GfBackend",
+                         "GfBackend",
+                         GF_TYPE_BACKEND,
                          G_PARAM_READWRITE |
                          G_PARAM_STATIC_STRINGS |
                          G_PARAM_CONSTRUCT_ONLY);
@@ -501,27 +514,25 @@ gf_monitor_init (GfMonitor *monitor)
   priv->mode_ids = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
-GfGpu *
-gf_monitor_get_gpu (GfMonitor *monitor)
+GfBackend *
+gf_monitor_get_backend (GfMonitor *monitor)
 {
   GfMonitorPrivate *priv;
 
   priv = gf_monitor_get_instance_private (monitor);
 
-  return priv->gpu;
+  return priv->backend;
 }
 
 void
 gf_monitor_make_display_name (GfMonitor *monitor)
 {
   GfMonitorPrivate *priv;
-  GfBackend *backend;
   GfMonitorManager *manager;
 
   priv = gf_monitor_get_instance_private (monitor);
 
-  backend = gf_gpu_get_backend (priv->gpu);
-  manager = gf_backend_get_monitor_manager (backend);
+  manager = gf_backend_get_monitor_manager (priv->backend);
 
   g_free (priv->display_name);
   priv->display_name = make_display_name (monitor, manager);
@@ -1061,16 +1072,11 @@ gf_monitor_calculate_mode_scale (GfMonitor                 *monitor,
                                  GfMonitorScalesConstraint  constraints)
 {
   GfMonitorPrivate *priv;
-  GfMonitorManager *monitor_manager;
-  GfBackend *backend;
   GfSettings *settings;
   gint global_scaling_factor;
 
   priv = gf_monitor_get_instance_private (monitor);
-  backend = gf_gpu_get_backend (priv->gpu);
-  monitor_manager = gf_backend_get_monitor_manager (backend);
-  backend = gf_monitor_manager_get_backend (monitor_manager);
-  settings = gf_backend_get_settings (backend);
+  settings = gf_backend_get_settings (priv->backend);
 
   if (gf_settings_get_global_scaling_factor (settings, &global_scaling_factor))
     return global_scaling_factor;
